function isDefined (item) {
    return ((typeof(item) === "undefined") ? false : true );
}
if (typeof Number.prototype.toRad == 'undefined') {
    Number.prototype.toRad = function() {
        return this * Math.PI / 180;
    }
}

var ZollInfo = new Class({

    nativeAppUriPrefix: 'http://reise-und-waren-20180611.zollapps.ch/' /*http://fugulier.ch/zollinfo/' /*'http://test1-reise-und-waren.zollapps.ch/'*/
    , language: 'de'

    , currentLanguage: null
    , validLanguages: ['de','fr','it','en']

    , uriMappings: {
        'de':['/','/de'],
        'fr':['/fr'],
        'it':['/it'],
        'en':['/en']
    }

    , offlineAlert: {
        'de':'Sie benötigen eine Internetverbindung, wenn Sie diese Version der App zum ersten mal benutzen.',
        'fr':'Connexion Internet requise pour le premier start de cette version de l\' application',
        'it':'Si prega di connettersi con internet',
        'en':'An internet connection is required to start if you use this version of the application for the first time'
    }

    //Minutes
    , dataMaxAge: 0

    , initialized: false
    , deviceReady: false

    , firstPageUriHash: 'root'

    , initialize: function() {
        //return;
        if(window.cordova!=null){
            document.addEventListener('deviceready', this.onDeviceReady.bind(this), false);
        }
        window.addEvent('domready:once', this.onDomReady.bind(this));
    }

    , onDeviceReady: function(){
        this.deviceReady = true
    }

    , onDomReady: function(){
        //Manifest defined
        var html = document.getElement('html');
        if(html && html.getAttribute('manifest')!=null){
            if(window.applicationCache && window.applicationCache.status>0 && window.applicationCache.addEventListener){
                var cache = window.applicationCache;
                cache.addEventListener('cached', this.waitCache.bind(this), false);
                //cache.addEventListener('checking', this.logEvent.bind(this), false);
                cache.addEventListener('downloading', function(){/*console.log('downloading manifest data')*/}, false);
                cache.addEventListener('error', this.waitCache.bind(this), false);
                cache.addEventListener('noupdate', this.waitCache.bind(this), false);
                cache.addEventListener('obsolete', this.waitCache.bind(this), false);
                //cache.addEventListener('progress', this.logEvent.bind(this), false);
                cache.addEventListener('updateready', this.waitCache.bind(this), false);
                return;
            }
        }

        //this.boot.delay(2000,this);
        this.boot();
    }

    , getCacheStatus: function(status){
        var appCache = window.applicationCache;
        switch (status) {
            case appCache.UNCACHED: // UNCACHED == 0
                return 'UNCACHED';
                break;
            case appCache.IDLE: // IDLE == 1
                return 'IDLE';
                break;
            case appCache.CHECKING: // CHECKING == 2
                return 'CHECKING';
                break;
            case appCache.DOWNLOADING: // DOWNLOADING == 3
                return 'DOWNLOADING';
                break;
            case appCache.UPDATEREADY:  // UPDATEREADY == 4
                return 'UPDATEREADY';
                break;
            case appCache.OBSOLETE: // OBSOLETE == 5
                return 'OBSOLETE';
                break;
            default:
                return 'UKNOWN CACHE STATUS';
                break;
        };
    }

    , waitCache: function(evt){
        //console.log(evt);
        this.boot();
    }

    , boot: function(){
        var language = this.language;
        //Try detect language if not set statically
        if(language==null){
            if(this.uriMappings!=null){
                var p = location.host.toLowerCase()+location.pathname.toLowerCase();
                var results = {};
                for(var k in this.uriMappings){
                    var arr = this.uriMappings[k];
                    for(var i=0;i<arr.length;i++){
                        var s = arr[i].toLowerCase();
                        if(p.indexOf(arr[i])!=-1){
                            results[p.indexOf(arr[i])+ s.length] = k;
                        }
                    }
                }

                var keys = Object.keys(results);
                if(keys.length>0){
                    keys.sort();
                    var k = keys[keys.length-1];
                    language = results[k];
                }
                else {
                    language = this.validLanguages[0];
                }
            }
        }
        this.setLanguage(language,false);
    }

    , setLanguage: function(language,redirect){

        if(language==null){
            //Get from cookie
            var lang = Cookie.read('language');
            //writeInfo('cookielang: '+lang);
            if(this.validLanguages.indexOf(lang)!=-1){
                language = lang;
            }
            //Get from browser
            if(language==null){
                language = this.validLanguages[0];
                if(navigator.language!=null){
                    var lang = navigator.language.substr(0,2).toLowerCase();
                    if(this.validLanguages.indexOf(lang)!=-1){
                        language = lang;
                    }
                }
            }
        }

        if(this.validLanguages.indexOf(language)!=-1){
            if(this.currentLanguage!=language){

                //Change icons to current language
                var links = document.id(document.head).getElements('link');
                for(var i=0;i<links.length;i++){
                    if(links[i].hasAttribute('rel')){
                        var rel = links[i].getAttribute('rel').toLowerCase();
                        if(rel.indexOf('icon')!=-1 && rel.indexOf('shortcut')==-1){
                            var href = links[i].getAttribute('href');
                            href = href.replace(/_[a-z]{2}\.png/,'_'+language+'.png');
                            links[i].setAttribute('href',href);
                        }
                    }
                }

                this.currentLanguage = language;

                Cookie.write('language', this.currentLanguage, {duration: 365});
                //writeInfo('set cookie: '+this.language);

                if(redirect){
                    window.addEvent('dataloaded:once',this.redirectToFirstPage.bind(this));
                }
                this.currentUriHash = null;
                this.loadData('data_'+this.currentLanguage);
            }
            else if(redirect){
                this.redirectToFirstPage();
            }
        }
    }

    , redirectToFirstPage: function(){
        this.loadPage(this.firstPageUriHash);
    }

    , getDataTimeStamp: function(){
        return this.dataTimeStamp;
    }

    , loadData: function(type){
        this.dataTimeStamp = 0;
        if(zollStorage.has('data_'+type+'_ts')){
            this.dataTimeStamp = zollStorage.get('data_'+type+'_ts');
        }
        var now = new Date().getTime();
        var diff = (now-this.dataTimeStamp)/1000/60;
        //writeInfo('Diff: '+Math.round((now-this.dataTimeStamp)/1000)+'secs '+navigator.onLine);
        //Not available or expired and online
        if(!zollStorage.has('data_'+type) || (diff>this.dataMaxAge && navigator.onLine)){
            //writeInfo('use remote');
            if(zollStorage.has('data_'+type+'_ts')!=null){
                this.dataTimeStamp = zollStorage.get('data_'+type+'_ts');
            }
            var url = (window.cordova!=null ? this.nativeAppUriPrefix : '')+'content/'+type+'.xml?mn='+now;
            var request = new Request({
                url: url
            }).addEvent('success', this.dataLoaded.bind(this,type)).addEvent('failure', this.dataLoaded.bind(this,type)).get();
        }
        else {
            //writeInfo('use local');
            this.dataLoaded(type,zollStorage.get('data_'+type));
        }
    }

    , dataLoaded: function(type,data,doc){
        var updateStorage = true;
        if(doc==null){
            if(zollStorage.has('data_'+type)){
                updateStorage = false;
                data = zollStorage.get('data_'+type);
            }
            //Fatal error
            else {
                //never been online
                alert(this.offlineAlert[this.currentLanguage]);
                return;
            }
        }

        if(updateStorage){
            this.dataTimeStamp = new Date().getTime();
            zollStorage.set('data_'+type+'_ts',this.dataTimeStamp);
            zollStorage.set('data_'+type, data);
        }

        //document.id('info').innerHTML = '';
        //writeInfo('data loaded');

        this.xml = null;
        var doc = null;
        if (window.ActiveXObject){
            doc = new ActiveXObject('Microsoft.XMLDOM');
            doc.async = false;
            doc.loadXML(data);
            var body = new Element('body');
            this.convertIEXmlDom(doc,body);
            this.xml = body.firstChild;
        }
        else {
            this.xml = new DOMParser().parseFromString(data, 'text/xml').firstChild;
        }

        //Add default ids to pages
        var currentId = 1;
        var pages = this.xml.getElements('page');
        for(var i=0;i<pages.length;i++){
            if(pages[i].get('id')==null){
                pages[i].set('id','_'+currentId+'_');
                currentId++;
            }
        }

        //Get first
        this.firstPageUriHash = this.xml.getElement('page').get('id');

        var body = document.id(document.body);
        body.getElements('.added_by_js').destroy();

        //Add template
        var templateNode = this.xml.getElement('template');
        if(!templateNode){
            alert('missing template node');
            return;
        }
        var elm = new Element('body', {'html': templateNode.getElement('body').get('text')});
        var children = elm.getChildren();
        for(var i=children.length-1;i>=0;i--){
            children[i].addClass('added_by_js');
            children[i].inject(body,'top');
        }

        this.container = document.id('pages');
        this.container.getChildren().destroy();

        //load all images
        var f = function(){
            var imgHtml = '';
            var pages = this.xml.getElements('page');
            if(pages){
                for(var i=0;i<pages.length;i++){
                    var imgName = this.getImageName(pages[i]);
                    if(imgName!=null && imgName.length>0){
                        var types = ['','_large'];
                        for(var j=0;j<types.length;j++){
                            var uri = 'static/pictos/'+imgName+types[j]+'.png';
                            imgHtml += uri+'<img src="'+uri+'"/><br>';
                        }
                    }
                }
            }
            var bodies = this.xml.getElements('body');
            if(bodies){
                var srcRegexp = /src=["']([^"']*)["']/i;
                for(var i=0;i<bodies.length;i++){
                    var imgMatches = bodies[i].get('text').match(/<img[^>]*>/gi);
                    if(imgMatches){
                        for(var j=0;j<imgMatches.length;j++){
                            var result = srcRegexp.exec(imgMatches[j]);
                            if(result && result[1]){
                                var uri = result[1];
                                imgHtml += uri+'<img src="'+uri+'"/><br>';
                                if(uri.indexOf('static/')!=-1){
                                    uri = uri.substring(uri.indexOf('static/'));
                                    imgHtml += uri+'<img src="'+uri+'"/><br>';
                                }
                            }
                        }
                    }
                }
            }
            if(imgHtml.length>0){
                var div = new Element('div',{'html':imgHtml, styles: {'position':'absolute', 'height':1, 'overflow':'hidden', 'top':0, 'left':-10000}});
                document.id(document.body).adopt(div);
            }
        };
        f.delay(1000,this);

        if(!this.initialized){
            //Setup
            if ('onhashchange' in window){
                window.onhashchange = this.onUriHashChange.bind(this);
                window.addEvent('hashchange', this.onUriHashChange.bind(this));
            }
            else {
                this.observeUriHash.bind(this).periodical(50);
            }

            this.initialized = true;
        }

        /* @todo reimplement
        document.id('pages').setStyle('min-height',body.getScrollHeight());
        */

        this.onUriHashChange();

        window.fireEvent('dataloaded');

        //writeInfo('loaded');
    }

    , getUriHash: function(){
        return this.cleanUriHash(self.location.hash);
    }

    , cleanUriHash: function(uriHash){
        if(uriHash.indexOf('#')==0){
            uriHash = uriHash.substr(1);
        }
        return uriHash;
    }

    , onUriHashChange: function(evt){
        this.container.setStyle('visibility','hidden');
        this.loadedUriHash = null;
        this.loadPage.delay(100,this,[this.getUriHash()]);
    }

    , observeUriHash: function(){
        var uriHash = this.getUriHash();
        if(uriHash==this.currentObserveUriHash){
            return;
        }
        this.container.setStyle('visibility','hidden');
        this.loadedUriHash = null;
        var oldUriHash = this.currentObserveUriHash;
        this.currentObserveUriHash = uriHash;
        window.fireEvent('hashchange', [oldUriHash, uriHash]);
    }

    , loadPage: function(uriHash){
        if(uriHash==='a_reda'){
            uriHash = 'root';
        }

        this.loadedUriHash = null;

        if(uriHash==this.currentUriHash){
            return;
        }

        document.getElements('form[role=\'search\']').getParent('.tools').removeClass('open');

        var xmlPage = this.xml.getElement('*[id='+uriHash+']');
        if(xmlPage==null){
            uriHash = this.firstPageUriHash;
            xmlPage = this.xml.getElement('*[id='+uriHash+']');
        }

        //Reload phonegap if we toggle between online/offline
        if(window.cordova!=null && this.deviceReady){
            if(uriHash==this.firstPageUriHash && this.currentUriHash==uriHash){
                if(navigator && navigator.app && navigator.app.exitApp){
                    try {
                        navigator.splashscreen.show();
                        navigator.app.exitApp();
                    }
                    catch(error){}
                }
                return;
            }

            if(window.navigator!=null && window.navigator.network!=null && window.navigator.network.connection!=null){
                if(this.wasOffline==null){
                    this.wasOffline = navigator.network.connection.type=='none';
                }
                else {
                    //Back to offline
                    if(this.wasOffline==false && navigator.network.connection.type=='none'){
                        this.wasOffline = true;
                    }
                    //Its the start page
                    if(this.wasOffline && navigator.network.connection.type!='none' && this.xml.getElement('page')==xmlPage){
                        self.location.href = self.location.href.split(/\?|#/)[0];
                        return;
                    }
                }
            }
        }

        this.container.setStyle('visibility','hidden');

        self.location.hash = uriHash;
        this.currentUriHash = uriHash;

        var page = xmlPage.retrieve('page');

        if(page){
            this.show(page);
            return;
        }

        var page = new ZollPage(this,uriHash,xmlPage);
        page.load();
    }

    , getCurrentPage: function() {
        var uriHash = this.getUriHash();
        var xmlPage = this.xml.getElement('page[id='+uriHash+']');
        if(xmlPage!=null){
            return xmlPage.retrieve('page');
        }
        return null;
    }

    , show: function(page){
        var uriHash = this.getUriHash();
        if(page && page.uriHash==uriHash){
            page.show();
            window.scrollTo.delay(0,window,[0,0]);
        }
        this.container.setStyle.delay(50,this.container,['visibility','visible']);
        this.loadedUriHash = uriHash;
    }

    , hideInactives: function(){
        var currentPage = this.getCurrentPage();
        var pages = this.container.getChildren();
        for(var i=0;i<pages.length;i++){
            var page = pages[i].retrieve('page');
            if(currentPage!=page){
                page.hide();
            }
        }
    }

    , getImageName: function(xmlPage){
        var imgName = xmlPage.get('picto');
        if(imgName==null){
            var nr = xmlPage.get('nr');
            if(nr!=null && nr.length>0){
                imgName = nr.replace(/\./g,'_');
            }
        }
        return imgName;
    }

    , translate: function(key, placeholders){
        var elm = document.getElement('.app_translations *[data-key=\''+key+'\']');
        if(elm){
            var txt = elm.get('text');
            if(placeholders){
                Object.each(placeholders,function(value,key){
                    if(value!=null){
                        txt = txt.replace('{'+key+'}',value);
                    }
                });
            }
            return txt;
        }
        return '?'+key;
    }

    , convertIEXmlDom : function(xmlNode,htmlParent){
        for (var i = 0; i < xmlNode.childNodes.length; i++){
            var child = xmlNode.childNodes[i];
            if(child.nodeName=='xml'){
                continue;
            }
            var elm = null;
            if(child.nodeType === 1){
                var attributes = {};
                if(child.attributes){
                    child.attributes.reset();
                    for (var a=0; a<child.attributes.length; a++) {
                        var attr = child.attributes[a];
                        attributes[attr.name] = attr.text;
                    }
                }
                elm = new Element(child.nodeName, attributes);
            } else {
                elm = document.createTextNode(child.data);
            }
            if(elm!=null){
                htmlParent.adopt(elm);
                this.convertIEXmlDom(child,elm);
            }
        }
    }
});

ZollPage = new Class({

    webapp: null
    , uri: null

    , isHidden: true

    , initialize: function(webapp,uriHash,xmlPage){
        this.webapp = webapp;
        this.uriHash = uriHash;
        this.xmlPage = xmlPage;
        this.xmlPage.store('page',this);
    }

    , load: function(){
        this.render();
        this.webapp.show(this);
    }

    , render: function() {
        //Already rendered
        if(this.elm){
            return;
        }

        this.container = document.id('pages');

        this.cssClassName = this.xmlPage.getAttribute('id').replace('/',' ');
        this.elm = new Element('div', {'class': 'content_container '+this.cssClassName});
        this.elm.store('page',this);

        this.elm.setStyle('display','none');
        this.elm.innerHTML = this.xmlPage.getElement('body').get('text');

        //Links
        if(!navigator.onLine){
            var links = this.elm.getElements('a').append(document.getElements('a'));
            for(var i=0;i<links.length;i++){
                if(links[i].get('href').indexOf('://')!=-1){
                    links[i].set('href','javascript:void(0)');
                    links[i].set('target','_self');
                }
            }
        }
        /*
        else if(window.cordova!=null){
            var links = this.elm.getElements('a').append(document.getElements('a'));
            for(var i=0;i<links.length;i++){
                if(links[i].get('href').indexOf('://')!=-1 && links[i].get('target')=='_blank'){
                    links[i].set('data-href',encodeURI(links[i].get('href')));
                    links[i].set('href','javascript:void(0)');
                    links[i].addEvent('click', function(){
                        window.open(this.get('data-href'), '_system', 'location=yes');
                    }.bind(links[i]));
                }
            }
        }
        */

        //onfocus="if(this._reset)return;this._reset=true;this.value=''" onclick="if(this._reset)return;this._reset=true;this.value=''"

        //Replace templates
        this.elm.getElements('div[data-template]').each(function(el){
            var selector = '.app_templates '+el.get('data-template').replace('@','');
            var templateElm = document.getElement(selector);
            if(templateElm){
                var newEl = templateElm.clone(true);
                newEl.replaces(el);
                for(var i=0;i<el.attributes.length;i++){
                    var name = el.attributes[i].nodeName;
                    if(name.indexOf('data-')!==-1){
                        newEl.set(name,el.get(name));
                    }
                }
                newEl.set('data-template',null);
                newEl.addClass('template');
                if(el.get('class')){
                    newEl.addClass(el.get('class'));
                }
            }
        }.bind(this));

        //Set rubric
        this.elm.getElements('.contentheader .info .text').each(function(el){
            var nr = this.xmlPage.get('nr');
            el.innerHTML = (nr==null ? '' : this.webapp.translate('rubric',{'nr':nr}));
        }.bind(this));

        //Set header image
        var imgElm = this.elm.getElement('.contentheader img');
        if(imgElm){
            var imgName = this.webapp.getImageName(this.xmlPage);
            if(imgName!=null && imgName.length>0){
                var uri = 'static/pictos/'+imgName+'_large.png';
                imgElm.set('src',uri);
            }
            else {
                imgElm.dispose();
            }
        }

        //Set backbutton parent
        var arr = this.xmlPage.get('id').split('/');
        if(arr.length>1){
            var parentPageUri = '#'+arr.slice(0,-1).join('/');
            this.elm.getElements('.back.button.parent').each(function(el){
                el.set('href',parentPageUri);
            }.bind(this));
        }
        else {
            this.elm.getElements('.back.button').each(function(el){
                el.setStyle('display','none');
            }.bind(this));
        }

        //Build inlinenavigation
        this.buildInlineNavigations();

        //Set time stamp
        var dataTimeStamp = '?';
        var ts = this.webapp.getDataTimeStamp();
        if(ts>0){
            var d = new Date();
            d.setTime(ts);
            dataTimeStamp = d.format('%d.%m.%Y');
        }
        this.elm.getElements('.lastmodified').each(function(el){
            el.innerHTML = dataTimeStamp;
        }.bind(this));
        //Change src if phonegap and static
        if(window.cordova!=null){
            this.elm.getElements('img').each(function(el){
                if(el.src.indexOf('static/')!=-1){
                    var src = el.src;
                    src = src.substring(src.indexOf('static/'));
                    el.src = src;
                }
            }.bind(this));
        }
        //Error handler
        this.elm.getElements('img').each(function(el){
            el.addEvent('error', function(){this.dispose();});
        }.bind(this));

        /*
        window.cordova = {};
        window.device = {};
        window.device.platform = 'ios';
        */
        if(window.cordova!=null && window.device!=null && window.device.platform!=null) {
            var os = window.device.platform.toLowerCase();
            if(os==='android' || os==='ios') {
                this.elm.getElements('a.quickzoll').each(function (elm) {
                    elm.set('href', elm.get('href') + '/'+os);
                });
            }
        }

        //Phone numbers
        new PhoneHelper(this.elm);

        //Search
        document.getElements('form[role=\'search\']').each(function (el) {
            if (el.get('data-init')) {
                return;
            }
            el.set('data-init', 'true');
            el.addEvent('submit', function () {
                var textElm = el.getElement('.text');
                if (textElm.getWidth() > 1) {
                    textElm.getParent('.tools').removeClass('open');
                    if (textElm.get('value').length > 0) {
                        //self.location.hash = el.get('data-pageid');
                        this.webapp.loadPage(el.get('data-pageid'));
                        this.webapp.getCurrentPage().search();
                    }
                }
                else {
                    textElm.getParent('.tools').addClass('open');
                }
                return false;
            }.bind(this));
        }.bind(this));
        this.search();
    }

    , handleRedirect: function(){
        window.clearTimeout(window.redirectTimer);
        window.redirectTimer = null;

        var nextUri = this.xmlPage.getAttribute('next-uri');
        if(nextUri){
            var parts = nextUri.split(';');
            var t = parts[1]*1;
            if(t<0 || isNaN(t)){
                t = 1000;
            }
            var f = function(){
                window.location.href = parts[0];
            }
            window.redirectTimer = window.setTimeout(f,t);
        }
    }

    , buildInlineNavigations: function(currentGeoPos){
        this.elm.getElements('.inlinenavigation').each(function(inlineNavigationElm){

            if(inlineNavigationElm.get('data-init')!=null){
                if(inlineNavigationElm.getElements('a[data-geo]').length==0){
                    return;
                }
            }

            inlineNavigationElm.set('data-init','done');

            if(this.inlineNavigationSectionItemElm==null){
                this.inlineNavigationSectionItemElm = inlineNavigationElm.getElement('.section').clone(false);
                this.inlineNavigationItemElm = inlineNavigationElm.getElement('.item').clone(true);
            }

            inlineNavigationElm.set('html','');


            var parent = this.xmlPage.getParent('pages');
            if(parent){
                var pages = null;
                if(inlineNavigationElm.getParent('.template.children')!=null){
                    pages = parent.getChildren('pages>page');
                }
                else {
                    pages = parent.getChildren('page');
                }
                if(pages){

                    var filter = null;
                    var filterElm = inlineNavigationElm.getParent('.template[data-filter]');
                    if(filterElm!=null){
                        filter = new RegExp(filterElm.get('data-filter'),'i');
                    }

                    var sort = false;
                    var elms = [];
                    for(var i=0;i<pages.length;i++){
                        if(filter!=null && !filter.test(pages[i].get('id'))){
                            continue;
                        }
                        if(pages[i]==this.xmlPage){
                            continue;
                        }

                        var itemElm = this.inlineNavigationItemElm.clone(true);
                        var geo = pages[i].get('geo');
                        var geoInfo = '';
                        if(geo!=null && geo.length>0){
                            itemElm.set('data-geo',geo);
                            var dist = 0;
                            if(currentGeoPos){
                                sort = true;
                                var parts = geo.split(';');
                                if(parts.length===2){
                                    //http://www.movable-type.co.uk/scripts/latlong.html
                                    dist = this.getGeoDistance(currentGeoPos,parts[0]*1,parts[1]*1);
                                    geoInfo = ' ('+this.roundGeoDistance(dist)+'km)';
                                }
                            }
                            itemElm.setAttribute('data-geodist',dist);
                        }
                        itemElm.set('href','#'+pages[i].get('id'));
                        var nr = pages[i].get('nr');
                        itemElm.getElement('span').set('html',(nr==null ? '' : this.webapp.translate('rubric',{'nr':nr})));
                        itemElm.getElement('strong').set('html',pages[i].getElement('title').get('text')+geoInfo);
                        var imgElm = itemElm.getElement('img');
                        if(imgElm){
                            var imgName = this.webapp.getImageName(pages[i]);
                            if(imgName!=null && imgName.length>0){
                                var uri = 'static/pictos/'+imgName+'.png';
                                imgElm.set('src',uri);
                            }
                            else {
                                imgElm.dispose();
                            }
                        }

                        elms.push(itemElm);
                    }

                    if(sort){
                        elms.sort(function(el1,el2){
                            return el1.getAttribute('data-geodist')-el2.getAttribute('data-geodist');
                        })
                    }

                    while(elms.length%2!==0){
                        elms.push(new Element('div',{'class':'item empty', 'html':'<div class="wrapper"></div>'}));
                    }

                    var currentSectionElm = null;
                    for(var i=0;i<elms.length;i++){
                        if(i%2==0){
                            currentSectionElm = this.inlineNavigationSectionItemElm.clone(true);
                            currentSectionElm.inject(inlineNavigationElm);
                        }
                        currentSectionElm.adopt(elms[i]);
                    }
                }
            }
        }.bind(this));
    }

    ,search: function(){
        var searchResultsElm = this.elm.getElement('.searchresults');
        if(!searchResultsElm){
            return;
        }
        var textElm = document.getElement('form .text');
        if(!textElm){
            return;
        }
        var query = textElm.get('value');
        if(this.currentSearchQuery==query){
            return;
        }
        this.currentSearchQuery = query;
        //console.log(searchResultsElm, query);
        searchResultsElm.set('html','');

        var results = [];
        if(query.length>=3){
            var weights = {'title':5, 'keywords':3, 'body':1};
            //weights = {'keywords':3};

            if(!this.webapp.searchIndex){
                this.webapp.searchIndex = [];
                this.webapp.xml.getElements('page').each(function(xmlPage){

                    var obj = {};
                    obj.xmlPage = xmlPage;
                    obj.searchable = {};

                    var xmlElm = xmlPage.getElement('body');
                    var elm = new Element('div');
                    elm.innerHTML = xmlElm.get('text');
                    var elms = elm.getElements('.keywords');
                    var value = '';
                    if(elms.length>0){
                        value = this.normalizeSearchValue(elms.get('text').toString().replace(/\s\s*/g,' ').trim())
                    }
                    //console.log(xmlPage.getElement('title').get('text'),value);
                    obj.searchable['keywords'] = value;

                    Object.keys(weights).each(function(which){
                        var xmlElm = xmlPage.getElement(which);
                        if(xmlElm && (!xmlElm.hasAttribute('class') || xmlElm.getAttribute('class').indexOf('nosearch')==-1)){
                            var elm = new Element('div');
                            elm.innerHTML = xmlElm.get('text');
                            elm.getElements('.nosearch').dispose();
                            var value = elm.get('text');
                            //console.log('value',value);
                            obj.searchable[which] = this.normalizeSearchValue(value.replace(/\s\s*/g,' ').trim());
                        }
                    }.bind(this));
                    this.webapp.searchIndex.push(obj);
                }.bind(this));
            }

            var regExp = new RegExp(this.normalizeSearchValue(query),'gi');
            for(var i=0;i<this.webapp.searchIndex.length;i++){
                var obj = {'xmlPage':this.webapp.searchIndex[i].xmlPage, 'rank':0};
                for(var which in this.webapp.searchIndex[i].searchable){
                    //console.log(this.webapp.searchIndex[i].searchable[which]);
                    var matches = this.webapp.searchIndex[i].searchable[which].match(regExp);
                    if(matches){
                        obj.rank += (weights[which]*matches.length);
                    }
                }
                if(obj.rank>0){
                    results.push(obj);
                }
            }
        }

        this.elm.getElements('.searchtherm').set('text',(query==null || query.length==0 ? '-' : query));

        if(results.length>0){
            results.sort(function(obj1,obj2){
                return obj2.rank-obj1.rank;
            });

            results.each(function(obj){
                var liElm = new Element('li');
                liElm.set('html','<a href="#'+obj.xmlPage.get('id')+'">'+obj.xmlPage.getElement('title').get('text')/*+' '+obj.rank*/+'</a>');
                searchResultsElm.adopt(liElm);
            }.bind(this));
        }
        else {
            searchResultsElm.set('html','<li>'+this.webapp.translate('no_search_results')+'</li>');
        }
    }

    , normalizeSearchValue: function(value){
        return value.toLowerCase();
    }

    , getLocation: function(){
        //writeInfo('get location');
        //this.onLocated({'coords':{'latitude':45.838846,'longitude':9.034882}})
        //this.onLocated({'coords':{'latitude':47.586715,'longitude':7.601166}})
        //this.onLocated({'coords':{'latitude':46.944106,'longitude':7.46727}})

        navigator.geolocation.getCurrentPosition(this.onLocated.bind(this));
    }

    , onLocated: function(pos){
        if(pos && pos.coords){

            this.buildInlineNavigations(pos);

            if(this.xmlPage.get('geo')){
                var parts = this.xmlPage.get('geo').split(';');
                if(parts.length===2){
                    var dist = this.getGeoDistance(pos,parts[0]*1,parts[1]*1);
                    var spans = this.elm.getElements('span.geodistance');
                    for(var i=0;i<spans.length;i++){
                        spans[i].innerHTML = '('+this.roundGeoDistance(dist)+'km)';
                    }
                }
            }

            if(this.getInterval==null && !this.isHidden){
                this.getLocation.delay(30000,this);
            }
        }
    }

    , roundGeoDistance: function(dist){
        var precision = 1;
        return Math.round(dist*precision)/precision;
    }

    , getGeoDistance: function(pos,lat,lon){
        if(pos && pos.coords){
            var R = 6371; // km
            var dLat = (lat-pos.coords.latitude).toRad();
            var dLon = (lon-pos.coords.longitude).toRad();
            var lat1 = pos.coords.latitude.toRad();
            var lat2 = lat.toRad();

            var a = Math.sin(dLat/2) * Math.sin(dLat/2) +
                Math.sin(dLon/2) * Math.sin(dLon/2) * Math.cos(lat1) * Math.cos(lat2);
            var c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a));
            return R * c;
        }
        return -1;
    }

    , show: function(){
        if(!this.isHidden){
            return;
        }
        this.isHidden = false;
        if(this.elm){
            document.title = this.xmlPage.getElement('title').get('text');
            var body = document.id(document.body);
            body.className = '';
            body.addClass(this.cssClassName);
            var homeElms = document.getElements('#mainwrapper a[href=#]');
            for(var i=0;i<homeElms.length;i++){
                if(homeElms[i].get('data-init')){
                    continue;
                }
                homeElms[i].set('data-init',true);
                homeElms[i].addEvent('click', function(e){
                    if(this.webapp.currentUriHash==this.webapp.firstPageUriHash){
                        e.stop();
                        return false;
                    }
                }.bind(this));
            }
            this.elm.inject(this.container,'bottom');
            this.elm.setStyle('display','');
            this.webapp.hideInactives();
        }

        var getLocation = false;
        var geos = this.elm.getElements('a[data-geo]');
        if(geos.length>0){
            getLocation = true;
        }
        if(this.xmlPage.get('geo') && this.elm.getElement('span.geodistance')){
            getLocation = true;
        }

        if(getLocation){
            if(navigator && navigator.geolocation && navigator.geolocation.getCurrentPosition){
                this.getLocation();
            }
        }

        this.handleRedirect();
    }

    , hide: function(){
        if(this.isHidden){
            return;
        }
        this.isHidden = true;
        if(this.elm){
            this.elm.dispose();
        }
    }

    , dispose: function(){
        this.elm.dispose();
        this.elm.destroy();
    }

});

var ZollStorage = new Class({
    initialize: function(version) {
        this.isAvailable = this.probe();
        if(this.isAvailable && isDefined(version) && this.get('___version')!=version){
            localStorage.clear();
            this.set('___version',version);
        }
    }
    , probe: function() {
        try {
            return isDefined(localStorage);
        }
        catch(error){}
        return false;
    }
    , set: function(key, value) {
        if (this.isAvailable) {
            localStorage[key] = value;
        }
    }
    , get: function(key) {
        if (this.isAvailable) {
            return isDefined(localStorage[key]) ? localStorage[key] : false;
        }
        return false;
    }
    , has: function(key) {
        return ( ( this.get(key) ) ? true : false );
    }
});

var PhoneHelper = new Class({
    Binds: ['makeCall', 'blur']

    , protocols: ['tel', 'sip', 'call', 'callto']
    , idle: null

    , element: null
    , iFrame: null
    , callStack: null
    , delay: null
    , number: null

    , timeout: 500

    , initialize: function(element){
        this.element = element;
        this.idle = true;
        this.attach();
    }

    , attach: function(){
        this.element.addEvent('click:relay(a[href^=tel:])', this.makeCall);
    }

    , detach: function(){
        this.element.removeEvent('click:relay(a[href^=tel:])', this.makeCall);
    }

    , makeCall: function(event,element){
        event.stop();
        var number = element.get('href').split(':')[1];
        if(this.idle && number.trim().length>0){
            element.addEvent('blur:once', this.blur);
            this.number = number;
            this.callStack = this.protocols.clone();
            this.idle = false;
            this.test();
        }
    }

    , blur: function(evt){
        this.destroyiFrame();
        clearTimeout(this.delay);
        this.idle = true;
    }

    , test: function(){
        this.destroyiFrame();
        var self = this;
        this.iFrame = new IFrame({
            src:this.callStack.shift()+':'+this.number,
            styles:{visibility:'hidden', position:'absolute'}
        });
        this.element.adopt(this.iFrame);
        this.time = new Date().getTime();
        this.delay = this.check.delay(this.timeout,this);
    }

    , check: function(){
        if(new Date().getTime() - this.time < (this.timeout+500) && this.callStack.length>0){
            this.test();
            return;
        }
        this.destroyiFrame();
        this.idle = true;
    }

    , destroyiFrame: function(){
        if(this.iFrame){
            this.iFrame.destroy();
            this.iFrame = null;
        }
    }
});

var zollStorage = new ZollStorage();
var zollInfo = new ZollInfo();
